indexing

	description: "Basic configuration file reader. Configuration files%
	%are text files with possible lines with comments in them.%
	%Comment line (start with # or ;) are skipped by this class."

	author: "Berend de Boer"
	date: "$Date: 2002/02/06 $"
	revision: "$Revision: #2 $"

class

	EPX_CONFIG_FILE


creation

	make


feature -- Initialization

	make (a_file_name: STRING) is
			-- Open `a_file_name' for reading.
		require
			file_name_not_empty: a_file_name /= Void and then not a_file_name.is_empty
		do
			create file.open_read (a_file_name)
			line_number := 0
		end


feature -- Public commands/queries

	eof: BOOLEAN is
		do
			Result := file.eof
		end

	last_line: STRING is
		do
			Result := file.last_string
		end

	line_number: INTEGER

	read_line is
			-- Read the next non-comment line.
		do
			from
				do_read_line
			until
				file.eof or else not is_comment
			loop
				do_read_line
			end
		end


feature {NONE} -- Implementation

	file: STDC_TEXT_FILE


feature {NONE} -- Private commands

	do_read_line is
		local
			c: CHARACTER
			i: INTEGER
			is_blank: BOOLEAN
		do
			line_number := line_number + 1
			file.read_line

			-- remove leading blanks
			from
				i := 1
				is_blank := True
			until
				not is_blank or else
				i > file.last_string.count
			loop
				c := file.last_string.item (i)
				is_blank := c = ' ' or else c = '%T'
				i := i + 1
			end
			if not is_blank and then i > 2 then
				file.last_string.keep_tail (i - 2)
			end

			-- remove trailing blanks
			from
				i := file.last_string.count
				is_blank := True
			until
				i = 0 or else
				not is_blank
			loop
				c := file.last_string.item (i)
				is_blank := c = ' ' or else c = '%T'
				i := i - 1
			end
			if not is_blank then
				file.last_string.keep_head (i + 1)
			end
		end

	is_comment: BOOLEAN is
			-- is last line read a comment
			-- empty lines count as comment too
		do
			Result :=
				file.last_string.count = 0 or else
			file.last_string.item (1) = '#' or else
			file.last_string.item (1) = ';'
		end


invariant

	file_is_open: file.is_open

end
