indexing

	description: "eposix's file and socket classes (STDC_FILE, POSIX_FILE_DESCRIPTOR, SUS_SOCKET) all inherit from this class. This makes eposix plug compatible with Gobo and any other library using a KI_CHARACTER_INPUT_STREAM."

	author: "Berend de Boer"
	date: "$Date: 2003/07/10 $"
	revision: "$Revision: #4 $"


deferred class

	EPX_CHARACTER_INPUT_STREAM


inherit

	KI_CHARACTER_INPUT_STREAM
		undefine
			close
		redefine
			is_rewindable,
			read_to_string
		end


feature -- Access

	is_rewindable: BOOLEAN is
			-- Can current input stream be rewound to return input from
			-- the beginning of the stream?
		once
			Result := True
		end

	is_streaming: BOOLEAN
			-- Is data coming through a network stream?

	last_string: STRING
			-- Last string read
			-- (Note: this query always return the same object.
			-- Therefore a clone should be used if the result
			-- is to be kept beyond the next call to this feature.
			-- However `last_string' is not shared between file objects.)


feature -- Stream or disk file

	set_streaming (enable: BOOLEAN) is
			-- Influence behaviour of certain functions if they should be
			-- optimized for data coming from disk or data coming from
			-- the network. In particular `is_streaming' implies that a
			-- client application is prepared to handle `read's that
			-- return less than the requested number of bytes, but don't
			-- assume that means end-of-file.
		do
			is_streaming := enable
		ensure
			streaming_set: is_streaming = enable
		end


feature -- Input

	last_read: INTEGER
			-- Last bytes read by `read_buffer'.
			-- Can be less than requested for non-blocking input.
			-- Check `last_blocked' in that case.

	read_buffer (buf: STDC_BUFFER; offset, nbytes: INTEGER) is
			-- Read data into `buf' at `offset' for `nbytes' bytes.
			-- Number of bytes actually read are available in `last_read'.
		require
			is_open_read: is_open_read
			not_end_of_input: not end_of_input
			nbytes_not_negative: nbytes >= 0
			offset_not_negative: offset >= 0
			buf_not_void: buf /= Void
			dont_overflow: buf.capacity >= offset + nbytes
		deferred
		ensure
			read_no_more_than_requested: last_read <= nbytes
		end

	read_to_string (a_string: STRING; pos, nb: INTEGER): INTEGER is
			-- Fill `a_string', starting at position `pos', with
			-- at most `nb' characters read from input stream.
			-- Return the number of characters actually read.
		local
			buf: STDC_BUFFER
		do
			create buf.allocate (nb)
			read_buffer (buf, 0, nb)
			Result := last_read
			buf.put_to_string (a_string, pos, 0, nb - 1)
		end


end
